package com.pancik.libgdx.zumpatalentov;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconDrawables;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.HashMap;
import java.util.Map;

public final class DrawableData {

    public static final String FONT_CHARACTERS = "abcdefghijklmnoprstquvwxyzABCDEFGHIJKLMNOPRSTQUVWXYZ1234567890+-*/()\\\"':;,.?![]{}_|<>@#$%^&~";
    public static SpriteBatch spriteBatch;
    public static Map<String, Texture> textures;
    public static BitmapFont whiteFont;
    public static BitmapFont blackFont;

    private static Float savedDensity = null;
    
    public static float getDensity() {
        if(savedDensity == null) {
            savedDensity = calculateDensity();
            return savedDensity;
        } else {
            return savedDensity;
        }
    }
    
    private static float calculateDensity() {
        float density = Gdx.graphics.getDensity();
        if (!MainApplicationListener.smallScreen) {
            if (density < 1.0f) {
                return 1.0f;
            } else {
                if (density == 1.0f) {
                    return density;
                } else {
                    for (int i = 0; i < 2; i++, density -= 0.5f) {
                        if (Gdx.graphics.getWidth() >= MainApplicationListener.SCREEN_WIDTH * (density - 0.1f) && Gdx.graphics.getHeight() >= MainApplicationListener.SCREEN_HEIGHT * (density - 0.1f)) {
                            return density;
                        }
                    }
                    return 1.0f;
                }
            }
        } else {
            return 1.0f;
        }
    }

    public static void setFontScale(float scale) {
        whiteFont.setScale(scale);
        blackFont.setScale(scale);
    }

    public static void loadData() {
        spriteBatch = new SpriteBatch();

        textures = new HashMap<String, Texture>();

        loadTextures();

        String filename = "data/drawable/font/Galindo-Regular.ttf";
        int size = (int) (25 * getDensity());

        FreeTypeFontGenerator generator = new FreeTypeFontGenerator(Gdx.files.internal(filename));

        blackFont = generator.generateFont(size, FONT_CHARACTERS, true);
        blackFont.setColor(Color.BLACK);
        blackFont.setScale(1.0f);
        //blackFont.setScale(getDensity());

        whiteFont = generator.generateFont(size, FONT_CHARACTERS, true);
        whiteFont.setColor(Color.WHITE);
        whiteFont.setScale(1.0f);
        //whiteFont.setScale(getDensity());

        generator.dispose();
    }

    private static void loadTextures() {
        DrawableData.textures.put("128x128", new Texture(Gdx.files.internal("data/drawable/buttons/128x128.png")));
        DrawableData.textures.put("128x128_down", new Texture(Gdx.files.internal("data/drawable/buttons/128x128_down.png")));
        DrawableData.textures.put("200x118", new Texture(Gdx.files.internal("data/drawable/buttons/200x118.png")));
        DrawableData.textures.put("200x118_down", new Texture(Gdx.files.internal("data/drawable/buttons/200x118_down.png")));
        DrawableData.textures.put("form", new Texture(Gdx.files.internal("data/drawable/ui/form.png")));
        
        DrawableData.textures.put("arrow_right", new Texture(Gdx.files.internal("data/drawable/ui/arrowRightNormal.png")));
        DrawableData.textures.put("arrow_right_down", new Texture(Gdx.files.internal("data/drawable/ui/arrowRightYellow.png")));
        DrawableData.textures.put("arrow_left", new Texture(Gdx.files.internal("data/drawable/ui/arrowLeftNormal.png")));
        DrawableData.textures.put("arrow_left_down", new Texture(Gdx.files.internal("data/drawable/ui/arrowLeftYellow.png")));
        DrawableData.textures.put("window_button_big", new Texture(Gdx.files.internal("data/drawable/ui/buttonNormal.png")));
        DrawableData.textures.put("window_button_big_down", new Texture(Gdx.files.internal("data/drawable/ui/buttonPressed.png")));
        DrawableData.textures.put("window_button_rectangle", new Texture(Gdx.files.internal("data/drawable/ui/buttonRectangle.png")));
        DrawableData.textures.put("window_button_rectangle_down", new Texture(Gdx.files.internal("data/drawable/ui/buttonRectanglePressed.png")));
        DrawableData.textures.put("window_button_x", new Texture(Gdx.files.internal("data/drawable/ui/buttonX.png")));
        DrawableData.textures.put("window_button_x_down", new Texture(Gdx.files.internal("data/drawable/ui/buttonXPressed.png")));
        
        DrawableData.textures.put("base_building_spot", new Texture(Gdx.files.internal("data/drawable/buildings/building_spot_B.png")));
        DrawableData.textures.put("frankie_factory", new Texture(Gdx.files.internal("data/drawable/buildings/frankie_factory.png")));
        DrawableData.textures.put("main_scientist", new Texture(Gdx.files.internal("data/drawable/buildings/main_scientist.png")));
        DrawableData.textures.put("mutate_lab", new Texture(Gdx.files.internal("data/drawable/buildings/mutate_lab.png")));
        DrawableData.textures.put("tesla_coil", new Texture(Gdx.files.internal("data/drawable/buildings/tesla_coil_3.png")));
        DrawableData.textures.put("tofu_tower", new Texture(Gdx.files.internal("data/drawable/buildings/tofu_tower_2.png")));
        DrawableData.textures.put("tower_building_spot", new Texture(Gdx.files.internal("data/drawable/buildings/building_spot_T.png")));
        DrawableData.textures.put("blood_explosion", new Texture(Gdx.files.internal("data/drawable/effects/blood_explosion.png")));
        DrawableData.textures.put("frankie_explosion", new Texture(Gdx.files.internal("data/drawable/effects/frankie_explosion.png")));
        DrawableData.textures.put("lightning_particle", new Texture(Gdx.files.internal("data/drawable/effects/lightning.png")));
        DrawableData.textures.put("nuke_explosion", new Texture(Gdx.files.internal("data/drawable/effects/explosion_3.png")));
        DrawableData.textures.put("plasma_explosion", new Texture(Gdx.files.internal("data/drawable/effects/plasma_explosion.png")));
        DrawableData.textures.put("purple_explosion", new Texture(Gdx.files.internal("data/drawable/effects/explosion_2.png")));
        DrawableData.textures.put("small_explosion", new Texture(Gdx.files.internal("data/drawable/effects/explosion_1.png")));
        DrawableData.textures.put("floor_tile", new Texture(Gdx.files.internal("data/drawable/floorTile.png")));
        DrawableData.textures.put("grass_tile", new Texture(Gdx.files.internal("data/drawable/grassTile.png")));
        DrawableData.textures.put("road_tile", new Texture(Gdx.files.internal("data/drawable/roadTile.png")));
        DrawableData.textures.put("wall_tile", new Texture(Gdx.files.internal("data/drawable/wallTile.png")));
        DrawableData.textures.put("star_icon", new Texture(Gdx.files.internal("data/drawable/icon/star_icon.png")));
        IconDrawables.init();
        DrawableData.textures.put("black_icon_circle", new Texture(Gdx.files.internal("data/drawable/ui/circle.png")));
        DrawableData.textures.put("tooltip", new Texture(Gdx.files.internal("data/drawable/ui/tooltip.png")));
        DrawableData.textures.put("window_background", new Texture(Gdx.files.internal("data/drawable/ui/background.png")));
        DrawableData.textures.put("window_well", new Texture(Gdx.files.internal("data/drawable/ui/well.png")));
        DrawableData.textures.put("laser_projectile", new Texture(Gdx.files.internal("data/drawable/effects/laser.png")));
        DrawableData.textures.put("plasma_projectile", new Texture(Gdx.files.internal("data/drawable/effects/plasma.png")));
        DrawableData.textures.put("frankie_move", new Texture(Gdx.files.internal("data/drawable/unit/frankie_move_2.png")));
        DrawableData.textures.put("frankie_attack", new Texture(Gdx.files.internal("data/drawable/unit/frankie_attack.png")));
        DrawableData.textures.put("batman_head", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/head_batman_move.png")));
        DrawableData.textures.put("bignose_head", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/head_bignose_move.png")));
        DrawableData.textures.put("bunny_head", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/head_bunny_move.png")));
        DrawableData.textures.put("melee_torso_move", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_melee_move.png")));
        DrawableData.textures.put("melee_torso_attack", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_melee_attack.png")));
        DrawableData.textures.put("onehand_torso_move", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_onehand_move.png")));
        DrawableData.textures.put("onehand_torso_attack", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_onehand_attack.png")));
        DrawableData.textures.put("twohand_torso_move", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_twohand_move.png")));
        DrawableData.textures.put("twohand_torso_attack", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_twohand_attack.png")));
        DrawableData.textures.put("four_legs", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/legs_four_move.png")));
        DrawableData.textures.put("three_legs", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/legs_three_move.png")));
        DrawableData.textures.put("two_legs", new Texture(Gdx.files.internal("data/drawable/unit/bodypart/legs_two_move.png")));
        
        DrawableData.textures.put("title", new Texture(Gdx.files.internal("data/drawable/title.png")));
        DrawableData.textures.put("title_notext", new Texture(Gdx.files.internal("data/drawable/title_notext.png")));
        DrawableData.textures.put("sound_on", new Texture(Gdx.files.internal("data/drawable/icon/sound_on.png")));
        DrawableData.textures.put("sound_off", new Texture(Gdx.files.internal("data/drawable/icon/sound_off.png")));
    }
    
    public static void unloadTextures() {
    }
    
    public static void renderMenuBackground() {
        if(!MainApplicationListener.smallScreen) {
            RenderUtils.blit(DrawableData.spriteBatch,
                    DrawableData.textures.get("title_notext"),
                    new RenderUtils.ScreenInfo(0, 0, Gdx.graphics.getWidth(), Gdx.graphics.getHeight()),
                    new RenderUtils.ScreenInfo(0, 0, 1920, 1080));
        } else {
            RenderUtils.blit(DrawableData.spriteBatch,
                    DrawableData.textures.get("title_notext"),
                    new RenderUtils.ScreenInfo(0, 0, 800, 480),
                    new RenderUtils.ScreenInfo(0, 0, 1920, 1080));
        }
    }
    
    public static void renderFrontScreen() {
        if(!MainApplicationListener.smallScreen) {
            RenderUtils.blit(DrawableData.spriteBatch,
                    DrawableData.textures.get("title"),
                    new RenderUtils.ScreenInfo(0, 0, Gdx.graphics.getWidth(), Gdx.graphics.getHeight()),
                    new RenderUtils.ScreenInfo(0, 0, 1920, 1080));
        } else {
            RenderUtils.blit(DrawableData.spriteBatch,
                    DrawableData.textures.get("title"),
                    new RenderUtils.ScreenInfo(0, 0, 800, 480),
                    new RenderUtils.ScreenInfo(0, 0, 1920, 1080));
        }
    }
}