package com.pancik.libgdx.zumpatalentov;

import com.badlogic.gdx.ApplicationListener;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.utils.ScissorStack;
import com.pancik.libgdx.zumpatalentov.screens.PickModeScreen;
import com.pancik.libgdx.zumpatalentov.screens.Screen;
import com.pancik.libgdx.zumpatalentov.util.InputData;
import com.pancik.libgdx.zumpatalentov.util.KeyChangedEvent;
import com.pancik.libgdx.zumpatalentov.util.KeyTypedEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseEventType;
import com.pancik.libgdx.zumpatalentov.util.MouseScrolledEvent;
import java.util.ArrayList;
import java.util.List;

public class MainApplicationListener implements ApplicationListener {
    public static final float PIXELS_PER_METER = 64.0f; 
    public static final int SCREEN_WIDTH = 800;
    public static final int SCREEN_HEIGHT = 480;

    
    private static Screen currentScreen = null;
    private OrthographicCamera currentCamera;
    private OrthographicCamera gameCamera;
    private OrthographicCamera uiCamera;
    private Rectangle glViewport;

    public static boolean smallScreen = false;
    private float accum = 0.0f;
    
    @Override
    public void create() {
        Gdx.input.setInputProcessor(new InputAdapter() {
            @Override
            public boolean touchDown(int x, int y, int pointer, int button) {
                Vector3 touchPointGame = new Vector3(x, y, 0);
                gameCamera.unproject(touchPointGame);
                Vector3 touchPointUI = new Vector3(x, y, 0);
                uiCamera.unproject(touchPointUI);

                InputData.mouseEvents.add(new MouseEvent((int)touchPointUI.x,
                        (int)touchPointUI.y,
                        (int)touchPointGame.x,
                        (int)touchPointGame.y,
                        MouseEventType.DOWN,
                        pointer));
                return true;
            }

            @Override
            public boolean touchUp(int x, int y, int pointer, int button) {
                Vector3 touchPointGame = new Vector3(x, y, 0);
                gameCamera.unproject(touchPointGame);
                Vector3 touchPointUI = new Vector3(x, y, 0);
                uiCamera.unproject(touchPointUI);

                InputData.mouseEvents.add(new MouseEvent((int)touchPointUI.x,
                        (int)touchPointUI.y,
                        (int)touchPointGame.x,
                        (int)touchPointGame.y,
                        MouseEventType.UP,
                        pointer));
                return true;
            }

            @Override
            public boolean touchDragged(int x, int y, int pointer) {
                Vector3 touchPointGame = new Vector3(x, y, 0);
                gameCamera.unproject(touchPointGame);
                Vector3 touchPointUI = new Vector3(x, y, 0);
                uiCamera.unproject(touchPointUI);

                InputData.mouseEvents.add(new MouseEvent((int)touchPointUI.x,
                        (int)touchPointUI.y,
                        (int)touchPointGame.x,
                        (int)touchPointGame.y,
                        MouseEventType.DRAG,
                        pointer));
                return true;
            }

            @Override
            public boolean scrolled(int amount) {
                InputData.mouseScrolledEvents.add(new MouseScrolledEvent(amount));
                return true;
            }
            
            @Override
            public boolean keyDown(int keycode) {
                //home - exits the app
                //back - goes back, specific for each screen
                //search - do nothing
                //menu - send as button press - we doesn't want this to be
                //       called as function because not everything uses this button
                switch(keycode) {
                    case Input.Keys.BACK:
                        currentScreen.handleBackButtonPressed();
                        break;
                    case Input.Keys.MENU:
                        currentScreen.handleMenuButtonPressed();
                        break;
                    default:
                        InputData.keyChangedEvents.add(new KeyChangedEvent(keycode, true));
                        break;
                }
                return true;
            }

            @Override
            public boolean keyUp(int keycode) {
                InputData.keyChangedEvents.add(new KeyChangedEvent(keycode,false));
                return true;
            }

            @Override
            public boolean keyTyped(char character) {
                InputData.keyTypedEvents.add(new KeyTypedEvent(character));
                return true;
            }
        });

        Gdx.input.setCatchBackKey(true);
        Gdx.input.setCatchMenuKey(true);

        DrawableData.loadData();
        SoundData.loadData();

        glViewport = new Rectangle(0, 0, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());

        gameCamera = new OrthographicCamera(SCREEN_WIDTH, SCREEN_HEIGHT);
        gameCamera.setToOrtho(true, SCREEN_WIDTH, SCREEN_HEIGHT);
        gameCamera.position.set(SCREEN_WIDTH/2, SCREEN_HEIGHT/2, 0);

        uiCamera = new OrthographicCamera(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
        uiCamera.setToOrtho(true, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
        uiCamera.position.set(Gdx.graphics.getWidth()/2, Gdx.graphics.getHeight()/2, 0);

        currentCamera = gameCamera;
        
        
        if(currentScreen == null) {
            currentScreen = new PickModeScreen(this);
        }
    }

    private static final float STEP_TIME = 1.0f/60.0f;
    
    @Override
    public void render() {
        //long startTimeTotal = TimeUtils.nanoTime();

        accum += Gdx.graphics.getDeltaTime();
        while (accum >= STEP_TIME) {
            currentScreen.tick();
            InputData.clear();
            accum -= STEP_TIME;
        }
        
        //before rendering update camera
        com.badlogic.gdx.graphics.GL10 gl = Gdx.graphics.getGL10();
        gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
        gl.glClearColor(0.5f, 0.5f, 0.5f, 1);
        gl.glViewport((int)this.glViewport.x,
                (int)this.glViewport.y,
                (int)this.glViewport.width,
                (int)this.glViewport.height);
        this.uiCamera.update();
        this.gameCamera.update();
        this.uiCamera.apply(gl);
        this.gameCamera.apply(gl);

        DrawableData.spriteBatch.setProjectionMatrix(currentCamera.combined);

        float frustrumBegx = gameCamera.frustum.planePoints[3].x;
        float frustrumBegy = gameCamera.frustum.planePoints[3].y;
        float frustrumEndx = gameCamera.frustum.planePoints[3].x - gameCamera.frustum.planePoints[1].x;
        if(frustrumEndx < 0) {
            frustrumEndx *= -1.0f;
        }
        float frustrumEndy = gameCamera.frustum.planePoints[1].y - gameCamera.frustum.planePoints[3].y ;
        if(frustrumEndy < 0) {
            frustrumEndy *= -1.0f;
        }        
        
        Rectangle scissors = new Rectangle();
        Rectangle clipBounds = new Rectangle(frustrumBegx, frustrumBegy, frustrumEndx, frustrumEndy);
        ScissorStack.calculateScissors(gameCamera, DrawableData.spriteBatch.getTransformMatrix(), clipBounds, scissors);
        ScissorStack.pushScissors(scissors);
        
        //render screen
        DrawableData.spriteBatch.begin();
        currentScreen.render();
        
        DrawableData.spriteBatch.flush();
        ScissorStack.popScissors();
        
        setCurrentCameraAndApplyMatrix(true);
        
        currentScreen.renderUI();
        
        /*
        RenderUtils.blitText(DrawableData.spriteBatch, "fps: " + Gdx.graphics.getFramesPerSecond(),
                ButtonHorizontalAlignment.RIGHT, -2,
                ButtonVerticalAlignment.BOTTOM, -2,
                RenderUtils.TextAlligment.RIGHT,
                RenderUtils.TextAlligment.TOP);
        */
        /*
        float x = Gdx.input.getX();
        float y = Gdx.input.getY();
        Vector3 unprojected = new Vector3(x, y, 0);
        uiCamera.unproject(unprojected);
        RenderUtils.blit(DrawableData.spriteBatch,
                TEXTURE_MOUSE_CURSOR,
                new RenderUtils.ScreenInfo(unprojected.x, unprojected.y, 16, 23),
                new RenderUtils.ScreenInfo(0, 0, 16, 23));
        */
        
        setCurrentCameraAndApplyMatrix(false);
        
        DrawableData.spriteBatch.end();

        /*
        if(Gdx.input.justTouched())
            Gdx.input.setOnscreenKeyboardVisible(true);
        */

        //System.out.println("Update: " + (TimeUtils.nanoTime() - startTimeTotal) / 1000000000.0f);
    }

    @Override
    public void pause() {
    }

    @Override
    public void resume() {
    }

    @Override
    public void dispose() {
        DrawableData.unloadTextures();
        
        SoundData.disposeData();
    }

    @Override
    public void resize(int width, int heigth) {
        if(Gdx.graphics.getWidth() < 800 || Gdx.graphics.getHeight() < 480) {
            smallScreen = true;

            uiCamera.setToOrtho(true, 800, 480);
            uiCamera.position.set(800/2, 480/2, 0);
        } else {
            smallScreen = false;

            gameCamera.setToOrtho(true, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
            
            uiCamera.setToOrtho(true, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
            uiCamera.position.set(Gdx.graphics.getWidth()/2, Gdx.graphics.getHeight()/2, 0);
        }

        glViewport = new Rectangle(0, 0, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());

        DrawableData.setFontScale(1.0f);
    }

    public void setScreen(Screen screen) {
    	MainApplicationListener.currentScreen = screen;
        this.gameCamera.zoom = 1.0f;
        Gdx.input.setOnscreenKeyboardVisible(false);
    }

    public void applyCameraMatrix() {
        this.currentCamera.update();
        //this.currentCamera.apply(Gdx.graphics.getGL10());
        DrawableData.spriteBatch.setProjectionMatrix(currentCamera.combined);
    }

    public void setCurrentCameraAndApplyMatrix(boolean setUiCamera) {
        if(setUiCamera) {
            this.currentCamera = uiCamera;
        } else {
            this.currentCamera = gameCamera;
        }
        applyCameraMatrix();
    }

    public void cameraZoom(float amount) {
        this.currentCamera.zoom += amount;
    }
    
    public OrthographicCamera getGameCamera() {
        return this.gameCamera;
    }
    
    public OrthographicCamera getUICamera() {
        return this.uiCamera;
    }
}