package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconButtonClickable;
import com.pancik.libgdx.zumpatalentov.engine.ui.IconSelection;
import com.pancik.libgdx.zumpatalentov.entity.Entity;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;

public abstract class BuildingSpotEntity extends Entity {
    protected Vector2 pos;
    protected Vector2 size;
    protected int team;
    protected Player owner;
    protected Engine.EngineControls engineControls;

    IconSelection iconSelection;
    
    public BuildingSpotEntity(Vector2 pos, Vector2 size, int team, Player owner, Engine.EngineControls engineControls) {
        this.pos = pos;
        this.size = size;
        this.team = team;
        this.owner = owner;
        this.engineControls = engineControls;
    }
    
    protected void initSelection(IconButtonClickable[] selection) {
        this.iconSelection = new IconSelection(this.pos, selection, engineControls);
    }
    
    protected void disableShowingSelection() {
        iconSelection.hide();
    }

    @Override
    public void tick() {
        buildingSpotTick();
    }
    
    public void renderUI() {
    }
    
    public boolean handle(MouseEvent me) {
        if(canBeBuiltOn()) {
            if(iconSelection.isHidden()) {
                if(me.mouseDown()) {
                    if(hitsTexture(me.getGameUnprojectedX(), me.getGameUnprojectedY())) {
                        owner.setIconSelectionAndShow(iconSelection);
                        return true;
                    }
                }
            }
        } else {
            return spotHandle(me);
        }
        
        return false;
    }
    
    public abstract boolean canBeBuiltOn();
    public abstract boolean spotHandle(MouseEvent me);
    public abstract boolean hitsTexture(int posx, int posy);
    public abstract void buildingSpotTick();
    
    public static class PossibilityCarrier {
        public String name;
        public ResourceManager.CostWrapper cost;
    }
    
    public abstract PossibilityCarrier[] getBuildingPossibilities();
    public abstract void tryBuiltPossibility(String s);
}
