package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.player.PlayablePlayer;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager;
import com.pancik.libgdx.zumpatalentov.engine.ui.window.MutateLabWindow;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.MutatedUnit;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.HeadBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.LegsBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.TorsoBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.Buff;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class MutateLabBuilding extends BaseBuilding {
    public static Texture MUTATE_LAB_TEXTURE = DrawableData.textures.get("mutate_lab");

    public static final int[] ANIMATION_STEPS = {
                                                    0,0,0,0,0,0,0,
                                                    1,1,1,1,1,1,1,
                                                    2,2,2,2,2,2,2,
                                                    3,3,3,3,3,3,3,
                                                    4,4,4,4,4,4,4,
                                                    5,5,5,5,5,5,5
                                                };
    
    public static final int TEX_SIZE_X = 256;
    public static final int TEX_SIZE_Y = 256;
    
    public static final float DEFAULT_SIZE_X = 192.0f;
    public static final float DEFAULT_SIZE_Y = 192.0f;
    
    public static final int SPAWN_COOLDOWN_TIME = 720;
    private static final int INCOME_PER_MINUTE = 40;
    protected static final int MAX_HEALTH = 500;
    
    protected int health = MAX_HEALTH;
    protected Engine.EngineControls engineControls;
    protected BaseBuildingSpot buildingSpot;
    
    protected boolean partsSet = false;
    protected HeadBodyPart.Type headType = null;
    protected TorsoBodyPart.Type torsoType = null;
    protected LegsBodyPart.Type legsType = null;
    
    protected MutateLabWindow window;
    
    protected int currentSpawnStep = 0;
    
    protected int animationStep = 0;
    
    public MutateLabBuilding(Vector2 pos, Vector2 size, int team, PlayablePlayer owner, Engine.EngineControls engineControls, BaseBuildingSpot buildingSpot) {
        super(pos, size, team, owner);
        this.engineControls = engineControls;
        this.buildingSpot = buildingSpot;
        
        this.window = new MutateLabWindow(ButtonHorizontalAlignment.MIDDLE, 0, ButtonVerticalAlignment.MIDDLE, 0, new MutateLabWindow.Callback() {

            @Override
            public boolean onPartsChosen(HeadBodyPart.Type head, TorsoBodyPart.Type torso, LegsBodyPart.Type legs) {
                //System.out.println("chosen: " + head + " t: " + torso + " l: " + legs);
                return chooseBodyParts(head, torso, legs);
            }

            @Override
            public Player getTooltipPlayer() {
                return MutateLabBuilding.this.owner;
            }

            @Override
            public void onBuildingDestroy() {
                MutateLabBuilding.this.buildingSpot.removeBuilding();
            }
        });
        this.window.hide();
        this.owner.setCurrentWindowAndShow(this.window);
        
        owner.getResourceManager().addIncomePerMinute(INCOME_PER_MINUTE);
    }

    @Override
    public void getHit(int damage, AttackableEntity from) {
        if(!isDead()) {
            health -= damage;
            
            PlayablePlayer playableOwner = (PlayablePlayer)owner;
            playableOwner.onBaseBuildingHit(health, MAX_HEALTH);
        }
    }

    @Override
    public boolean isDead() {
        return (health <= 0);
    }

    @Override
    public void tick() {
        if(!isDead()) {
            if(partsSet) {
                animationStep++;

                if(currentSpawnStep >= (SPAWN_COOLDOWN_TIME * owner.getUpgradeCoefficient(Buff.PercentageStat.SPAWN_SPEED))) {
                    HeadBodyPart head = HeadBodyPart.getHeadBodyPart(headType);
                    TorsoBodyPart torso = TorsoBodyPart.getTorsoBodyPart(torsoType, engineControls);
                    LegsBodyPart legs = LegsBodyPart.getTorsoBodyPart(legsType);

                    UnitEntity newMutatedUnit = new MutatedUnit(owner.getSpawnPosition(), MutatedUnit.getDefaultSize(), team, owner, engineControls, head, torso, legs);
                    newMutatedUnit.setWaypointPath(owner.getRandomWaypointPath());
                    engineControls.addEntity(newMutatedUnit);
                    currentSpawnStep = 0;
                }
                currentSpawnStep++;
            }
            
            Vector3 unprojectedPos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0.0f);
            engineControls.getUICamera().unproject(unprojectedPos);
            window.tick(unprojectedPos.x, unprojectedPos.y);
        }
    }

    public boolean chooseBodyParts(HeadBodyPart.Type head, TorsoBodyPart.Type torso, LegsBodyPart.Type legs) {
        ResourceManager.CostWrapper costHead = head.getCost();
        ResourceManager.CostWrapper costTorso = torso.getCost();
        ResourceManager.CostWrapper costLegs = legs.getCost();
        
        int coinsAmount = costHead.coinsAmount + costTorso.coinsAmount + costLegs.coinsAmount;
        int bonusAmount = costHead.bonusAmount + costTorso.bonusAmount + costLegs.bonusAmount;
        
        if(owner.getResourceManager().hasCurrencyFor(coinsAmount, bonusAmount)) {
            owner.getResourceManager().removeCurrency(coinsAmount, bonusAmount);
            
            if(owner.isLocal()) {
                SoundData.playRandomSound("coins_1", "coins_2");
            }
            
            headType = head;
            torsoType = torso;
            legsType = legs;
            partsSet = true;
            
            return true;
        } else {
            owner.addErrorMessage("Insufficient funds for this combination.");
            
            return false;
        }
    }
    
    @Override
    public void render() {
        if(!isDead()) {
            RenderUtils.blit(DrawableData.spriteBatch,
                    MUTATE_LAB_TEXTURE,
                    new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                    new RenderUtils.ScreenInfo(TEX_SIZE_X * ANIMATION_STEPS[animationStep % ANIMATION_STEPS.length], 0, TEX_SIZE_X, TEX_SIZE_Y));
        }
    }

    @Override
    public boolean remove() {
        return isDead();
    }
    
    public boolean isProducing() {
        return partsSet;
    }

    @Override
    public void onRemove() {
        owner.getResourceManager().addIncomePerMinute(-INCOME_PER_MINUTE);
    }
    
    @Override
    public boolean hitsBuilding(int x, int y) {
        if(x >= pos.x - size.x/2 && x <= pos.x + size.x/2 && y >= pos.y - size.y/2 && y <= pos.y + size.y/2) {
            return true;
        } else {
            return false;
        }
    }
    
    @Override
    public boolean handle(MouseEvent me) {
        boolean doHandle = hitsBuilding(me.getGameUnprojectedX(), me.getGameUnprojectedY()) && me.mouseDown() && !isDead();

        if (doHandle) {
            owner.setCurrentWindowAndShow(window);
        }

        return doHandle;
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }

    @Override
    public void forceRemove() {
        health = 0;
    }

    @Override
    public boolean showSellMenu() {
        return false;
    }
}
