package com.pancik.libgdx.zumpatalentov.engine.player.manager;

public class ResourceManager {
    public static class CostWrapper {
        public int coinsAmount;
        public int bonusAmount;

        public CostWrapper(int coinsAmount, int bonusAmount) {
            this.coinsAmount = coinsAmount;
            this.bonusAmount = bonusAmount;
        }
    }
    
    public static final int INCOME_ADD_STEPS = 60;
    
    private int coinsAmount = 35;
    private int incomePerMinute = 0;

    private int bonusCurrencyAmount = 0;
    
    private int step = 0;
    private float incomeToAdd = 0.0f;
    
    public void tick() {
        coinsAmount += (int)incomeToAdd;
        incomeToAdd -= (int)incomeToAdd;
        
        step++;
        if(step >= INCOME_ADD_STEPS) {
            incomeToAdd += ((float)INCOME_ADD_STEPS * (float)incomePerMinute) / 3600.0f;
            step = 0;
        }
    }
    
    public int getCoinsAmount() {
        return coinsAmount;
    }

    public int getIncomePerMinute() {
        return incomePerMinute;
    }

    public int getBonusCurrencyAmount() {
        return bonusCurrencyAmount;
    }
    
    public void addIncomePerMinute(int deltaIncomePerMinute) {
        this.incomePerMinute += deltaIncomePerMinute;
    }
    
    public boolean hasCoinsFor(int amount) {
        return (coinsAmount >= amount);
    }
    
    public boolean hasBonusCurrencyFor(int amount) {
        return (bonusCurrencyAmount >= amount);
    }
    
    public boolean hasCurrencyFor(int coins, int bonus) {
        return (hasCoinsFor(coins) && hasBonusCurrencyFor(bonus));
    }
    
    public boolean hasCurrencyFor(CostWrapper cost) {
        return hasCurrencyFor(cost.coinsAmount, cost.bonusAmount);
    }
    
    public void addCoins(int amount) {
        this.coinsAmount += amount;
    }

    public void addBonusCurrency(int amount) {
        this.bonusCurrencyAmount += amount;
    }

    public void removeCoins(int amount) {
        this.coinsAmount -= amount;
    }

    public void removeBonusCurrency(int amount) {
        this.bonusCurrencyAmount -= amount;
    }
    
    public void removeCurrency(int coins, int bonus) {
        removeCoins(coins);
        removeBonusCurrency(bonus);
    }
    
    public void removeCurrency(CostWrapper cost) {
        removeCurrency(cost.coinsAmount, cost.bonusAmount);
    }
}
