package com.pancik.libgdx.zumpatalentov.engine.player.tutorial;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class TextTutorial extends Tutorial {
    protected static enum State {
        MOVING,
        WRITING;
    }
    
    protected static final float CAMERA_SPEED = 10.0f;
    
    protected Vector2 target;
    protected String[] text;

    protected Vector2 cameraPos;
    protected State state;
    protected int currentTextAmount = 1;
    protected boolean finished = false;
    
    public TextTutorial(EngineControls engineControls, Vector2 target, String[] text) {
        super(engineControls);
        this.target = target;
        this.text = text;
        this.state = State.MOVING;
    }
    
    @Override
    public void tick() {
        switch(state) {
            case MOVING:
                if(this.cameraPos == null) {
                    this.cameraPos = new Vector2(engineControls.getGameCamera().position.x, engineControls.getGameCamera().position.y);
                }
                Vector2 dir = target.cpy();
                dir.sub(cameraPos);
                if(dir.len() > CAMERA_SPEED) {
                    dir.nor();
                    dir.mul(CAMERA_SPEED);
                }
                this.cameraPos.add(dir);
                
                engineControls.getGameCamera().position.set(cameraPos.x, cameraPos.y, 0.0f);
                if(cameraPos.dst(target) <= 1.0f) {
                    state = State.WRITING;
                }
                break;
            case WRITING:
                engineControls.getGameCamera().position.set(cameraPos.x, cameraPos.y, 0.0f);
                break;
        }
    }

    @Override
    public void renderUI() {
        switch(state) {
            case MOVING:
                break;
            case WRITING:
                int startX = -150;
                for(int i = 0; i < currentTextAmount; i++) {
                    RenderUtils.blitText(DrawableData.spriteBatch,
                            text[i],
                            ButtonHorizontalAlignment.MIDDLE,
                            0,
                            ButtonVerticalAlignment.BOTTOM,
                            startX,
                            RenderUtils.TextAlligment.MIDDLE,
                            RenderUtils.TextAlligment.MIDDLE);
                    
                    startX += 30;
                }
                break;
        }
    }

    @Override
    public void handle(MouseEvent me) {
        if(me.mouseDown()) {
            switch(state) {
                case MOVING:
                    if(this.cameraPos == null) {
                        this.cameraPos = new Vector2(engineControls.getGameCamera().position.x, engineControls.getGameCamera().position.y);
                    }
                    this.cameraPos.set(target);

                    state = State.WRITING;
                    break;
                case WRITING:
                    currentTextAmount++;
                    if(currentTextAmount > text.length) {
                        finished = true;
                    }
                    break;
            }
        }
    }
    
    @Override
    public boolean isFinished() {
        return finished;
    }
}
