package com.pancik.libgdx.zumpatalentov.engine.tile;

import com.badlogic.gdx.graphics.Texture;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class RoadTile extends Tile {
    public static Texture ROAD_TILE_TEXTURE = DrawableData.textures.get("road_tile");
    public static final int TEX_WIDTH = 128;
    public static final int TEX_HEIGHT = 128;
    
    private int imageIndex = 0;
    private Tile neighbourTile = null;
    
    public RoadTile(int posx, int posy) {
        super(posx, posy, Type.ROAD);
    }

    @Override
    public void tick() {
    }

    @Override
    public void render() {
        //blit anything that should be under this tile
        if (neighbourTile != null) {
            if (neighbourTile.getUnderlyingTexture() != null) {
                RenderUtils.blit(DrawableData.spriteBatch,
                        neighbourTile.getUnderlyingTexture(),
                        new RenderUtils.ScreenInfo(posx - TILE_SIZE / 2,
                        posy - TILE_SIZE / 2,
                        Tile.TILE_SIZE,
                        Tile.TILE_SIZE),
                        new RenderUtils.ScreenInfo(TEX_WIDTH * (imageIndex % 4), TEX_HEIGHT * (imageIndex / 4), TEX_WIDTH, TEX_HEIGHT));
            }
        }
        
        //blit road
        RenderUtils.blit(DrawableData.spriteBatch,
                ROAD_TILE_TEXTURE,
                new RenderUtils.ScreenInfo(posx - TILE_SIZE/2,
                    posy - TILE_SIZE/2,
                    Tile.TILE_SIZE,
                    Tile.TILE_SIZE),
                new RenderUtils.ScreenInfo(TEX_WIDTH * (imageIndex % 4), TEX_HEIGHT * (imageIndex / 4), TEX_WIDTH, TEX_HEIGHT));
    }

    @Override
    public void setTextureImageIndex(int index) {
        this.imageIndex = index;
    }

    @Override
    public void setNeighbourTile(Tile t) {
        this.neighbourTile = t;
    }

    @Override
    public Texture getUnderlyingTexture() {
        return null;
    }
    
    @Override
    public boolean isWalkable() {
        return true;
    }
}
