package com.pancik.libgdx.zumpatalentov.engine.ui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RandomGenerator;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class BonusCurrencyItem {
    public static Texture STAR_TEXTURE = DrawableData.textures.get("star_icon");
    
    public static interface CallbackHandler {
        public void onCurrencyAdded();
    }
    
    protected static int STAR_TEX_SIZE_X = 128;
    protected static int STAR_TEX_SIZE_Y = 128;
    
    protected static float ITEM_SIZE_X = 48.0f;
    protected static float ITEM_SIZE_Y = 48.0f;
    protected static float ITEM_SIZE_FLYING_X = 24.0f;
    protected static float ITEM_SIZE_FLYING_Y = 24.0f;
    
    private static final float INTERPOLATION_STEP_SIZE = 0.075f;
    private static final int INTERPOLATION_STEPS = 15;
    private static final float DELTA_MAX = 4.0f;
    private static final float DELTA_MIN = -4.0f;
    private static final float DELTA_PER_FRAME = 0.2f;
    
    private static final int BONUS_ITEM_LIFETIME = 600;
    
    private static final float TOLERATED_LENGHT = 10.0f;

    protected Icon icon;
    protected Vector2 originPos;
    protected Vector2 groundPos;
    protected Engine.EngineControls engineControls;
    protected BonusCurrencyItem.CallbackHandler callback;
    
    protected boolean adding = false;
    protected float currentDelta = 0.0f;
    
    protected int offsetX;
    protected Vector2 currentPos;
    protected Vector2 posUI;
    protected int currentFallingStep = 0;
    protected Vector2 mousePos;
    protected boolean clicked = false;
    protected int currentLifetime = 0;
    
    protected boolean flying = false;
    protected boolean finishedFlying = false;
    protected float currentSizeX = ITEM_SIZE_X;
    protected float currentSizeY = ITEM_SIZE_Y;

    public BonusCurrencyItem(Icon icon, Vector2 originPos, Vector2 groundPos, Engine.EngineControls engineControls, BonusCurrencyItem.CallbackHandler callback) {
        this.icon = icon;
        this.originPos = originPos;
        this.groundPos = groundPos;
        this.engineControls = engineControls;
        this.callback = callback;
        
        this.currentPos = originPos.cpy();
        this.posUI = new Vector2();
        Vector3 projectedPos = new Vector3(this.originPos.x, this.originPos.y, 0.0f);
        engineControls.getGameCamera().project(projectedPos);
        projectedPos.y = Gdx.graphics.getHeight() - projectedPos.y;
        engineControls.getUICamera().unproject(projectedPos);
        this.posUI.set(projectedPos.x, projectedPos.y);
        
        this.offsetX = RandomGenerator.getRandom(200);
        this.offsetX -= 100;
    }

    public void tick() {
        currentLifetime++;
        currentFallingStep++;
        
        if(currentFallingStep / INTERPOLATION_STEPS < 3) {
            Vector2 targetPos = new Vector2(offsetX, -50.0f);
            targetPos.add(originPos);
            if(currentFallingStep / INTERPOLATION_STEPS != 0) {
                //targetPos.set(10.0f, 25.0f);
                targetPos.set(groundPos).add(offsetX, 0.0f);
            }

            float mu2 = (1.0f - (float)Math.cos(INTERPOLATION_STEP_SIZE * 3.14))/2.0f;
            currentPos.set(currentPos.x * (1-mu2) + targetPos.x * mu2, currentPos.y * (1-mu2) + targetPos.y * mu2);
        }
    }
    
    public void render() {
        if(!remove()) {
            if(!flying) {
                Vector3 projectedPos = new Vector3(this.currentPos.x, this.currentPos.y, 0.0f);
                engineControls.getGameCamera().project(projectedPos);
                projectedPos.y = Gdx.graphics.getHeight() - projectedPos.y;
                engineControls.getUICamera().unproject(projectedPos);
                this.posUI.set(projectedPos.x, projectedPos.y);
                
                if(currentFallingStep / INTERPOLATION_STEPS >= 3) {
                    if(adding) {
                        currentDelta += DELTA_PER_FRAME;

                        if(currentDelta >= DELTA_MAX) {
                            currentDelta = DELTA_MAX;
                            adding = false;
                        }
                    } else {
                        currentDelta -= DELTA_PER_FRAME;

                        if(currentDelta <= DELTA_MIN) {
                            currentDelta = DELTA_MIN;
                            adding = true;
                        }
                    }

                    posUI.y += currentDelta;
                }
            } else {
                Vector2 target = new Vector2(14, 40);
                
                posUI.lerp(target, 0.05f);
                if(target.dst(posUI) < TOLERATED_LENGHT && !finishedFlying) {
                    callback.onCurrencyAdded();
                    finishedFlying = true;
                }
                
                target.set(ITEM_SIZE_FLYING_X, ITEM_SIZE_FLYING_Y);
                Vector2 size = new Vector2(currentSizeX, currentSizeY);
                size.lerp(target, 0.05f);
                currentSizeX = size.x;
                currentSizeY = size.y;
            }
            
            //set position
            
            if(!flying) {
                DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.8f);

                RenderUtils.blit(DrawableData.spriteBatch,
                        STAR_TEXTURE,
                        new RenderUtils.ScreenInfo(posUI.x - ITEM_SIZE_X/1.5f - 5, posUI.y - ITEM_SIZE_Y/1.5f - 3, ITEM_SIZE_X *1.5f, ITEM_SIZE_Y * 1.5f),
                        new RenderUtils.ScreenInfo(0, 0, STAR_TEX_SIZE_X, STAR_TEX_SIZE_Y));
            }
            
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.8f);
            
            boolean hits = false;
            if(mousePos != null) {
                if(hitsIcon((int)mousePos.x, (int)mousePos.y)) {
                    hits = true;
                    DrawableData.spriteBatch.setColor(0.5f, 0.5f, 0.5f, 0.8f);
                }
            }

            icon.render((int)posUI.x, (int)posUI.y, (int)currentSizeX, (int)currentSizeY);

            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
    }
    
    public void forcePickup() {
        callback.onCurrencyAdded();
        flying = true;
        finishedFlying = true;
    }
    
    public boolean hitsIcon(int posx, int posy) {
        if(posx >= posUI.x - ITEM_SIZE_X/2 && posx <= posUI.x + ITEM_SIZE_X/2 && posy >= posUI.y - ITEM_SIZE_Y/2 && posy <= posUI.y + ITEM_SIZE_Y/2) {
            return true;
        } else {
            return false;
        }
    }
    
    public boolean remove() {
        if(!flying) {
            return (clicked || (currentLifetime) >= BONUS_ITEM_LIFETIME);
        } else {
            return finishedFlying;
        }
    }
    
    public boolean handleUIMouse(MouseEvent me) {
        if(!remove() && !flying) {
            if(me.mouseDown()) {
                if(hitsIcon(me.getUiUnprojectedX(), me.getUiUnprojectedY())) {
                        SoundData.playSound("tofu_squash");
                        flying = true;
                        clicked = true;
                        return true;
                    }
            }
            /*
            if(me.mouseDown()) {
                mousePos = new Vector2(me.getUiUnprojectedX(), me.getUiUnprojectedY());
            } else if(me.mouseDragged()) {
                if(mousePos != null) {
                    mousePos.set(me.getUiUnprojectedX(), me.getUiUnprojectedY());
                }
            } else if(me.mouseUp()) {
                if(mousePos != null) {
                    mousePos = null;

                    if(hitsIcon(me.getUiUnprojectedX(), me.getUiUnprojectedY())) {
                        flying = true;
                        clicked = true;
                        return true;
                    }
                }
            }
            */
        }
        return false;
    }
}
