package com.pancik.libgdx.zumpatalentov.engine.ui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class IconSelection {
    public static Texture BLACK_MIDDLE_CIRCLE_TEXTURE = DrawableData.textures.get("black_icon_circle");
    
    public static final float MAX_CIRCLE_DIAMETER_MULTIPLIER = 13.0f;
    private float circleDiameter = 0.0f;
    private boolean showSelection = false;

    protected Vector2 pos;
    protected IconButtonClickable[] selection;
    protected Engine.EngineControls engineControls;

    public IconSelection(Vector2 pos, IconButtonClickable[] selection, EngineControls engineControls) {
        this.pos = pos;
        this.selection = selection;
        this.engineControls = engineControls;
    }
    
    public void tick() {
        if(showSelection) {
            for(IconButtonClickable icon : selection) {
                Vector3 unprojectedPos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0.0f);
                engineControls.getUICamera().unproject(unprojectedPos);

                icon.tick(unprojectedPos.x, unprojectedPos.y);
            }
        }
    }
    
    public void renderUI() {
        if(showSelection) {
            circleDiameter += getMaxCircleDiameter() / 10.0f;
            if(circleDiameter > getMaxCircleDiameter()) {
                circleDiameter = getMaxCircleDiameter();
            }

            Vector3 projectedPos = new Vector3(this.pos.x, this.pos.y, 0.0f);
            engineControls.getGameCamera().project(projectedPos);
            projectedPos.y = Gdx.graphics.getHeight() - projectedPos.y;
            engineControls.getUICamera().unproject(projectedPos);
            
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 0.75f);
            
            RenderUtils.blit(DrawableData.spriteBatch,
                BLACK_MIDDLE_CIRCLE_TEXTURE,
                new RenderUtils.ScreenInfo(projectedPos.x - circleDiameter, projectedPos.y - circleDiameter, circleDiameter * 2, circleDiameter * 2),
                new RenderUtils.ScreenInfo(0, 0, 256, 256));
            
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
            
            //render icons
            Vector2 iconPos = new Vector2(0.0f, -circleDiameter);
            
            float anglePerIcon = 360.0f/((float)selection.length);
            
            for(IconButtonClickable icon : selection) {
                icon.setPos(new Vector2(projectedPos.x, projectedPos.y).add(iconPos));
                icon.render();
                
                iconPos.rotate(anglePerIcon);
            }
        }
    }
    
    protected float getMaxCircleDiameter() {
        return MAX_CIRCLE_DIAMETER_MULTIPLIER * selection.length;
    }
    
    public boolean handle(MouseEvent me) {
        if(showSelection) {
            boolean doHandle = true;
            boolean hitsAnyIcon = false;
            for(IconButtonClickable icon : selection) {
                if(doHandle) {
                    if(!icon.handleUIMouse(me)) {
                        doHandle = false;
                    }
                }

                if(icon.hitsIcon(me.getUiUnprojectedX(), me.getUiUnprojectedY())) {
                    hitsAnyIcon = true;
                }
            }

            if(hitsAnyIcon) {
                return true;
            }
        }
        
        return false;
    }
    
    public boolean isHidden() {
        return !showSelection;
    }
    
    public void hide() {
        showSelection = false;
        for(IconButtonClickable icon : selection) {
            icon.resetInput();
        }
    }
    
    public void show() {
        circleDiameter = 0.0f;
        showSelection = true;
    }
}
