package com.pancik.libgdx.zumpatalentov.engine.ui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class Tooltip {
    public static Texture TOOLTIP_TEXTURE = DrawableData.textures.get("tooltip");
    
    protected static final int TEX_SIZE_X = 512;
    protected static final int TEX_SIZE_Y = 256;
    protected static final int CORNER_SIZE_X = 64;
    protected static final int CORNER_SIZE_Y = 64;
    
    protected float posx;
    protected float posy;
    protected int coinsCost;
    protected int bonusCost;
    protected String title;
    protected String[] desc;

    public Tooltip(float posx, float posy, int coinsCost, int bonusCost, String title, String[] desc) {
        this.posx = posx;
        this.posy = posy;
        this.coinsCost = coinsCost;
        this.bonusCost = bonusCost;
        this.title = title;
        this.desc = desc;
    }
    
    public Tooltip(int coinsCost, int bonusCost, String title, String[] desc) {
        this(0, 0, coinsCost, bonusCost, title, desc);
    }
    
    public Tooltip(ResourceManager.CostWrapper cost, String title, String[] desc) {
        this(0, 0, cost.coinsAmount, cost.bonusAmount, title, desc);
    }
    
    public void render() {
        renderUITooltip(posx, posy, coinsCost, bonusCost, title, desc);
    }
    
    public void setPos(float posx, float posy) {
        this.posx = posx;
        this.posy = posy;
    }
    
    public static void renderUITooltip(float posx, float posy, int coinsCost, int bonusCost, String title, String[] desc) {
        BitmapFont.TextBounds size = DrawableData.blackFont.getBounds(coinsCost + "");
        float coinsSizeX = size.width;
        size = DrawableData.blackFont.getBounds(bonusCost + "");
        float bonusSizeX = size.width;
        
        //sizeX
        float costSizeX = coinsSizeX + bonusSizeX + 20 + 16 + 16 + 5;
        DrawableData.setFontScale(0.8f);
        size = DrawableData.blackFont.getBounds(title);
        float titleSizeX = size.width + 5;
        float descSizeX = -1.0f;
        DrawableData.setFontScale(0.7f);
        for(String s : desc) {
            size = DrawableData.blackFont.getBounds(s);
            if(descSizeX < size.width) {
                descSizeX = size.width;
            }
        }
        DrawableData.setFontScale(1.0f);
        float maxSizeX = Math.max(Math.max(costSizeX, titleSizeX), descSizeX + 5.0f);
        if(maxSizeX < 224.0f) {
            maxSizeX = 224.0f;
        }
        maxSizeX += 32;
        
        //sizeY
        float sizeY;
        
                
        float maxSizeY = 35.0f;
        if(coinsCost > 0 || bonusCost > 0) {
            maxSizeY += 30.0f;
        }
        if(desc.length > 0) {
            maxSizeY += 8.0f;
        }
        maxSizeY += 15.0f * desc.length;
        
        posx = Gdx.graphics.getWidth() - maxSizeX - 5.0f;
        posy = Gdx.graphics.getHeight() - maxSizeY - 5.0f;
        
        /*
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx, posy, 256.0f, 128.0f),
                new RenderUtils.ScreenInfo(0, 0, TEX_SIZE_X, TEX_SIZE_Y));
        */
        
        //corners
        //top left
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + 1.0f, posy + 1.0f, 32.0f, 32.0f),
                new RenderUtils.ScreenInfo(0, 0, CORNER_SIZE_X, CORNER_SIZE_Y));
        
        //top right
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + maxSizeX - 32.0f - 1.0f, posy + 1.0f, 32.0f, 32.0f),
                new RenderUtils.ScreenInfo(TEX_SIZE_X - CORNER_SIZE_X, 0, CORNER_SIZE_X, CORNER_SIZE_Y));
        
        //bottom left
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + 1.0f, posy + maxSizeY - 32.0f - 1.0f, 32.0f, 32.0f),
                new RenderUtils.ScreenInfo(0, TEX_SIZE_Y - CORNER_SIZE_Y, CORNER_SIZE_X, CORNER_SIZE_Y));
        
        //bottom right
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + maxSizeX - 32.0f - 1.0f, posy + maxSizeY - 32.0f - 1.0f, 32.0f, 32.0f),
                new RenderUtils.ScreenInfo(TEX_SIZE_X - CORNER_SIZE_X, TEX_SIZE_Y - CORNER_SIZE_Y, CORNER_SIZE_X, CORNER_SIZE_Y));
        
        //corner to corner fill
        //top
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + 32.0f, posy + 1.0f, maxSizeX - 32.0f - 32.0f, 32.0f),
                new RenderUtils.ScreenInfo(CORNER_SIZE_X, 0, TEX_SIZE_X - CORNER_SIZE_X * 2, CORNER_SIZE_Y));
        
        //left
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + 1.0f, posy + 32.0f, 32.0f, maxSizeY - 32.0f - 32.0f),
                new RenderUtils.ScreenInfo(0, CORNER_SIZE_Y, CORNER_SIZE_X, TEX_SIZE_Y - CORNER_SIZE_Y * 2));
        
        //right
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + maxSizeX - 32.0f - 1.0f, posy + 32.0f, 32.0f, maxSizeY - 32.0f - 32.0f),
                new RenderUtils.ScreenInfo(TEX_SIZE_X - CORNER_SIZE_X, CORNER_SIZE_Y, CORNER_SIZE_X, TEX_SIZE_Y - CORNER_SIZE_Y * 2));
        
        //bottom
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + 32.0f, posy + maxSizeY - 32.0f - 1.0f, maxSizeX - 32.0f - 32.0f, 32.0f),
                new RenderUtils.ScreenInfo(CORNER_SIZE_X, TEX_SIZE_Y - CORNER_SIZE_Y, TEX_SIZE_X - CORNER_SIZE_X * 2, CORNER_SIZE_Y));
        
        //fill the rect in middle
        RenderUtils.blit(DrawableData.spriteBatch,
                TOOLTIP_TEXTURE,
                new RenderUtils.ScreenInfo(posx + 32.0f, posy + 32.0f, maxSizeX - 32.0f - 32.0f, maxSizeY - 32.0f -32.0f),
                new RenderUtils.ScreenInfo(CORNER_SIZE_X, CORNER_SIZE_Y, TEX_SIZE_X - CORNER_SIZE_X * 2, TEX_SIZE_Y - CORNER_SIZE_Y * 2));
        
        
        int startX = (int)posx + 7;
        int startY = (int)posy + 5;
        
        if(coinsCost > 0 || bonusCost > 0) {
            if(coinsCost > 0) {
                RenderUtils.blitText(DrawableData.spriteBatch,
                        coinsCost + "",
                        startX,
                        startY,
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.BOTTOM);

                startX = (int)(startX + coinsSizeX) + 16;
                Icon.COIN.render(startX, startY + 12, 24, 24);
                startX += 20;
            }

            if(bonusCost > 0) {
                RenderUtils.blitText(DrawableData.spriteBatch,
                        bonusCost + "",
                        startX,
                        startY,
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.BOTTOM);

                startX = (int)(startX + bonusSizeX) + 16;
                Icon.TOFU.render(startX, startY + 12, 24, 24);
            }

            startX = (int)posx + 5;
            startY += 30;
        }
        
        DrawableData.setFontScale(0.8f);
        RenderUtils.blitText(DrawableData.spriteBatch,
                title,
                startX,
                startY,
                new Color(0.7f, 0.7f, 0.0f, 1.0f),
                RenderUtils.TextAlligment.LEFT,
                RenderUtils.TextAlligment.BOTTOM);
        
        startY += 8;
        DrawableData.setFontScale(0.7f);
        for(String s : desc) {
            startY += 15;
            RenderUtils.blitText(DrawableData.spriteBatch,
                    s,
                    startX,
                    startY,
                    Color.WHITE,
                    RenderUtils.TextAlligment.LEFT,
                    RenderUtils.TextAlligment.BOTTOM);
        }
        
        DrawableData.setFontScale(1.0f);
    }
}
