package com.pancik.libgdx.zumpatalentov.engine.ui.window;

import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.engine.ui.Tooltip;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonImageEnum;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;

public class WindowCooldownButton extends WindowButton {
    public static interface Callback {
        public boolean onButtonPressed();
    }
    
    protected boolean onCooldown = false;
    protected int cooldownStep = 0;
    protected final int cooldownTime;
    
    protected Player tooltipPlayer;
    protected Tooltip tooltip;
    protected Icon icon;
    protected Callback cooldownCallback;
    
    public WindowCooldownButton(ButtonHorizontalAlignment horizontal, int deltax,
            ButtonVerticalAlignment vertical, int deltay,
            String text, ButtonImageEnum imageType,
            Callback cooldownCallback,
            int offsetX, int offsetY,
            int sizeX, int sizeY,
            int cooldownTime, Tooltip tooltip, Player tooltipPlayer) {
        super(horizontal, deltax, vertical, deltay, text, imageType, null, offsetX, offsetY, sizeX, sizeY);
        this.cooldownTime = cooldownTime;
        this.icon = null;
        this.cooldownCallback = cooldownCallback;
        this.tooltip = tooltip;
        this.tooltipPlayer = tooltipPlayer;
    }
    
    public WindowCooldownButton(Icon icon, ButtonHorizontalAlignment horizontal, int deltax,
            ButtonVerticalAlignment vertical, int deltay,
            String text, ButtonImageEnum imageType,
            Callback cooldownCallback,
            int offsetX, int offsetY,
            int sizeX, int sizeY,
            int cooldownTime, Tooltip tooltip, Player tooltipPlayer) {
        this(horizontal, deltax, vertical, deltay, text, imageType, cooldownCallback, offsetX, offsetY, sizeX, sizeY, cooldownTime, tooltip, tooltipPlayer);
        this.icon = icon;
    }

    @Override
    public void tick(float mousePosX, float mousePosY, boolean hidden) {
        super.tick(mousePosX, mousePosY, hidden);
        
        if(checkIfClicked((int)mousePosX, (int)mousePosY) && !hidden) {
            if(tooltip != null) {
                tooltip.setPos(mousePosX, mousePosY);
                tooltipPlayer.setTooltip(tooltip);
            }
        }
        
        if(onCooldown) {
            if(cooldownTime > 0) {
                cooldownStep++;
                //System.out.println("tick cd " + cooldownStep);
                if(cooldownStep >= cooldownTime) {
                    cooldownStep = 0;
                    onCooldown = false;
                }
            }
        }
    }
    
    @Override
    public void render() {
        if(!onCooldown) {
            super.render();
        } else {
            super.renderClicked();
        }
    }
    
    @Override
    public void render(int x, int y, int sizeX, int sizeY) {
        super.render(x, y, sizeX, sizeY);
        
        int textDeltay = 0;
        if((checkIfClicked(mousePos.x, mousePos.y) && mouseClicked) || onCooldown) {
            textDeltay = 1;
        }
        
        if(icon != null) {
            icon.render(calculateX() + imageType.getDensitySizex()/2, calculateY() + imageType.getDensitySizey()/2 + textDeltay,
                    imageType.getDensitySizex(), imageType.getDensitySizey());
        }
    }

    @Override
    public void click() {
        if(!onCooldown) {
            if(cooldownCallback.onButtonPressed()) {
                if(cooldownTime >= 0) {
                    onCooldown = true;
                    cooldownStep = 0;
                }
            }
        }
    }

    @Override
    public void handle(MouseEvent me) {
        if(!onCooldown) {
            super.handle(me);
        }
    }
}
