package com.pancik.libgdx.zumpatalentov.entity.projectile;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.particle.PlasmaExplosionParticle;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class PlasmaHomingProjectile extends HomingProjectile {
    public static Texture PLASMA_TEXTURE = DrawableData.textures.get("plasma_projectile");
    
    public static final int TEX_SIZE_X = 64;
    public static final int TEX_SIZE_Y = 64;
    public static final float DIAMETER = 16.0f;
    public static final float SPEED = 4.0f;
    
    public static final int[] ANIMATION_STEPS = {0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3};
    
    int currentStep = 0;
    
    protected Engine.EngineControls engineControls;
    
    public PlasmaHomingProjectile(AttackableEntity target, Vector2 pos, Callback callback, Engine.EngineControls engineControls) {
        super(target, pos, DIAMETER, SPEED, callback);
        this.engineControls = engineControls;
    }
    
    @Override
    public void render() {
        RenderUtils.blit(DrawableData.spriteBatch,
                PLASMA_TEXTURE,
                new RenderUtils.ScreenInfo(pos.x - diameter/2, pos.y - diameter/2, diameter, diameter),
                new RenderUtils.ScreenInfo(TEX_SIZE_X * ANIMATION_STEPS[currentStep % ANIMATION_STEPS.length], 0, TEX_SIZE_X, TEX_SIZE_Y), angle);
        currentStep++;
    }

    @Override
    public void onRemove() {
    }

    @Override
    protected void onProjectileArrived() {
        engineControls.addParticle(new PlasmaExplosionParticle(pos, diameter));
    }
}
