package com.pancik.libgdx.zumpatalentov.entity.unit;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.Level.WaypointPath;
import com.pancik.libgdx.zumpatalentov.engine.particle.FrankieExplosionParticle;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.entity.WaypointAI;
import com.pancik.libgdx.zumpatalentov.entity.WaypointAttackAI;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.Buff;
import com.pancik.libgdx.zumpatalentov.util.RandomGenerator;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class FrankieUnit extends UnitEntity {
    public static Texture MUMMY_MOJAM_TEXTURE = DrawableData.textures.get("frankie_move");
    public static Texture MUMMY_MOJAM_ATTACK_TEXTURE = DrawableData.textures.get("frankie_attack");
    public static int TEX_SIZE = 128;
    public static int ATTACK_TEXT_SIZE = 128;
    protected static final int[] ANIMATION_STEPS = {0,0,0,0,0,0,0, 1,1,1,1,1,1,1, 2,2,2,2,2,2,2, 3,3,3,3,3,3,3};
    protected static final int[] ATTACK_ANIMATION_STEPS = {0,0,0,0,0,0,0,
                                                           1,1,1,1,1,1,1,
                                                           2,2,2,2,2,2,2,
                                                           3,3,3,3,3,3,3,
                                                           2,2,2,2,2,2,2,
                                                           1,1,1,1,1,1,1};
    
    protected static final int ANIMATION_INDEX_LEFT = 0;
    protected static final int ANIMATION_INDEX_UP = 1;
    protected static final int ANIMATION_INDEX_RIGHT = 2;
    protected static final int ANIMATION_INDEX_DOWN = 3;
    
    protected static final float UNIT_SPEED = 2.0f;
    protected static final int UNIT_HEALTH = 100;
    
    protected static final int ATTACK_COOLDOWN = ATTACK_ANIMATION_STEPS.length;
    protected static final int ATTACK_DAMAGE = 10;
    protected static final float ATTACK_SIGHT_RANGE = 250.0f;
    
    protected static final float DEFAULT_SIZE_X = 64.0f;
    protected static final float DEFAULT_SIZE_Y = 64.0f;
    
    protected static final float BONUS_CURRENCY_DROP_CHANCE = 0.25f;
    
    protected float animationCurrentStep = 0.0f;
    protected int selectedAnimation = ANIMATION_INDEX_DOWN;
    
    protected int health;
    
    public FrankieUnit(Vector2 pos, Vector2 size, int team, Player owner, Engine.EngineControls engineControls) {
        super(pos, size, team, owner, engineControls);
        
        this.ai = new WaypointAttackAI(engineControls);
        setAi(ai);
        
        this.health = (int)(UNIT_HEALTH * getBuffCoefficient(Buff.PercentageStat.HEALTH_AMOUNT));
    }
    
    @Override
    protected void unitTick() {
    }

    @Override
    public void unitRender() {
        if(!isDead()) {
            if(!attacked || ATTACK_ANIMATION_STEPS.length <= attackCdStep) {
                RenderUtils.blit(DrawableData.spriteBatch,
                        MUMMY_MOJAM_TEXTURE,
                        new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                        new RenderUtils.ScreenInfo(TEX_SIZE * ANIMATION_STEPS[(int)animationCurrentStep], TEX_SIZE * selectedAnimation, TEX_SIZE, TEX_SIZE));
            } else {
                RenderUtils.blit(DrawableData.spriteBatch,
                        MUMMY_MOJAM_ATTACK_TEXTURE,
                        new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                        new RenderUtils.ScreenInfo(ATTACK_TEXT_SIZE * ATTACK_ANIMATION_STEPS[attackCdStep],
                                                   ATTACK_TEXT_SIZE * selectedAnimation, ATTACK_TEXT_SIZE, ATTACK_TEXT_SIZE));
            }
        }
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
    }
    
    @Override
    protected void moveUp() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_UP;

            this.pos.add(0, -getSpeed());
        }
    }

    @Override
    protected void moveDown() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_DOWN;

            this.pos.add(0, getSpeed());
        }
    }

    @Override
    protected void moveRight() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_RIGHT;

            this.pos.add(getSpeed(), 0);
        }
    }

    @Override
    protected void moveLeft() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_LEFT;

            this.pos.add(-getSpeed(), 0);
        }
    }
    
    protected void moveAnimationForward() {
        animationCurrentStep += 1.0f * getBuffCoefficient(Buff.PercentageStat.MOVEMENT_SPEED);
        if((int)animationCurrentStep >= ANIMATION_STEPS.length) {
            animationCurrentStep = 0.0f;
        }
    }

    protected float getSpeed() {
        return UNIT_SPEED * getBuffCoefficient(Buff.PercentageStat.MOVEMENT_SPEED);
    }
    
    @Override
    public void setWaypointPath(WaypointPath waypointPath) {
        if(ai instanceof WaypointAI) {
            ((WaypointAI)ai).setWaypointPath(waypointPath);
        }
    }

    @Override
    public MoveResult[] getMoveResults() {
        int[] x = { 1, -1, 0, 0 };
        int[] y = { 0, 0, -1, 1};
        MoveType[] types = {MoveType.MOVE_RIGHT, MoveType.MOVE_LEFT, MoveType.MOVE_UP, MoveType.MOVE_DOWN};
        
        assert ((x.length == y.length) && (x.length == types.length));
        
        List<MoveResult> moveResults = new ArrayList<MoveResult>();
        
        for(int i = 0; i < x.length; i++) {
            Vector2 cpyPos = this.pos.cpy();
            cpyPos.add(getSpeed() * x[i], getSpeed() * y[i]);
            
            MoveResult temp = new MoveResult();
            temp.move = types[i];
            temp.result = cpyPos;
            moveResults.add(temp);
        }
        
        MoveResult[] out = new MoveResult[moveResults.size()];
        moveResults.toArray(out);
        return out;
    }

    @Override
    public void apply(MoveType type) {
        switch(type) {
            case MOVE_DOWN:
                tryMoveDown();
                break;
            case MOVE_LEFT:
                tryMoveLeft();
                break;
            case MOVE_RIGHT:
                tryMoveRight();
                break;
            case MOVE_UP:
                tryMoveUp();
                break;
            default:
                try {
                    throw new Exception("Missing handler for movement type " + type);
                } catch (Exception ex) {
                    Logger.getLogger(FrankieUnit.class.getName()).log(Level.SEVERE, null, ex);
                }
        }
    }

    @Override
    protected int getAttackCooldown() {
        return (int)(ATTACK_COOLDOWN * getBuffCoefficient(Buff.PercentageStat.ATTACK_SPEED));
    }
    
    @Override
    public float getAttackRange() {
        return MELEE_ATTACK_RANGE;
    }
    
    @Override
    public float getAttackSightRange() {
        return ATTACK_SIGHT_RANGE;
    }

    @Override
    protected boolean attack(AttackableEntity e) {
        if(!isDead()) {
            if (e.getPos().dst(pos)  <= MELEE_ATTACK_RANGE) {
                SoundData.playSound("punch", 0.5f);
                
                e.getHit((int)(ATTACK_DAMAGE * getBuffCoefficient(Buff.PercentageStat.ATTACK_POWER)), this);
                
                Vector2 dir = e.getPos();
                dir.sub(pos);
                float lenx = dir.x;
                if(lenx < 0) {
                    lenx *= -1.0f;
                }
                float leny = dir.y;
                if(leny < 0) {
                    leny *= -1.0f;
                }
                if(lenx > leny) {
                    if(dir.x < 0) {
                        selectedAnimation = ANIMATION_INDEX_LEFT;
                    } else {
                        selectedAnimation = ANIMATION_INDEX_RIGHT;
                    }
                } else {
                    if(dir.y < 0) {
                        selectedAnimation = ANIMATION_INDEX_DOWN;
                    } else {
                        selectedAnimation = ANIMATION_INDEX_UP;
                    }
                }
                
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    @Override
    public void getHit(int damage, AttackableEntity from) {
        if(!isDead()) {
            this.health -= damage;
            
            if(isDead()) {
                int chance = 1 + RandomGenerator.getRandom(100);
                if((float)chance / 100.0f <= BONUS_CURRENCY_DROP_CHANCE) {
                    from.getOwner().getBonusManager().addReward(this.getPos(), this.getPos().add(0.0f, size.y/2));
                }
                
                engineControls.addParticle(new FrankieExplosionParticle(pos, size.x));
                
                SoundData.playRandomSound("guts_1", "guts_2");
            }
        }
    }

    @Override
    public boolean isDead() {
        if(health <= 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }

    @Override
    public float getDiameter() {
        return Math.min(size.x, size.y)/2.5f;
    }
}
