package com.pancik.libgdx.zumpatalentov.entity.unit;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.Level.WaypointPath;
import com.pancik.libgdx.zumpatalentov.engine.particle.BloodExplosionParticle;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.entity.WaypointAI;
import com.pancik.libgdx.zumpatalentov.entity.WaypointAttackAI;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.HeadBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.LegsBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.TorsoBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.Buff;
import com.pancik.libgdx.zumpatalentov.util.RandomGenerator;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class MutatedUnit extends UnitEntity {
    /*
    protected static final int[] ANIMATION_STEPS = {0,0,0,0,0,0,0, 1,1,1,1,1,1,1, 2,2,2,2,2,2,2, 3,3,3,3,3,3,3};
    protected static final int[] ATTACK_ANIMATION_STEPS = {0,0,0,0,0,0,0,
                                                           1,1,1,1,1,1,1,
                                                           2,2,2,2,2,2,2,
                                                           3,3,3,3,3,3,3,
                                                           2,2,2,2,2,2,2,
                                                           1,1,1,1,1,1,1};
    */
    
    protected static final int ANIMATION_INDEX_LEFT = 0;
    protected static final int ANIMATION_INDEX_UP = 1;
    protected static final int ANIMATION_INDEX_RIGHT = 2;
    protected static final int ANIMATION_INDEX_DOWN = 3;
    
    protected static final int UNIT_HEALTH = 100;
    
    //protected static final float ATTACK_SIGHT_RANGE = 250.0f;
    
    protected static final float DEFAULT_SIZE_X = 128.0f;
    protected static final float DEFAULT_SIZE_Y = 128.0f;
    
    protected static final float BONUS_CURRENCY_DROP_CHANCE = 0.25f;
    
    protected double animationCurrentStep = 0.0f;
    protected int selectedAnimation = ANIMATION_INDEX_DOWN;
    
    protected int health;
    
    protected HeadBodyPart head;
    protected TorsoBodyPart torso;
    protected LegsBodyPart legs;
    
    public MutatedUnit(Vector2 pos, Vector2 size, int team, Player owner, Engine.EngineControls engineControls, HeadBodyPart head, TorsoBodyPart torso, LegsBodyPart legs) {
        super(pos, size, team, owner, engineControls);
        this.head = head;
        this.torso = torso;
        this.legs = legs;
        
        this.ai = new WaypointAttackAI(engineControls);
        setAi(ai);
        
        this.health = (int) (head.getHealth() * getBuffCoefficient(Buff.PercentageStat.HEALTH_AMOUNT));
    }
    
    @Override
    protected void unitTick() {
    }

    @Override
    public void unitRender() {
        if(!isDead()) {
            /*
            RenderUtils.blit(DrawableData.spriteBatch,
                    GrassTile.GRASS_TILE_TEXTURE,
                    new RenderUtils.ScreenInfo(pos.x - getDiameter()/2, pos.y - getDiameter()/2, getDiameter(), getDiameter()),
                    new RenderUtils.ScreenInfo(0, 0, getDiameter(), getDiameter()));
            */
            
            if(!attacked) {
                legs.render(pos, size, (int)animationCurrentStep, selectedAnimation, attacked);
                head.render(pos, size, (int)animationCurrentStep, selectedAnimation, attacked);
                torso.render(pos, size, (int)animationCurrentStep, selectedAnimation, attacked);
            } else {
                legs.render(pos, size, (int)animationCurrentStep, selectedAnimation, attacked);
                head.render(pos, size, (int)animationCurrentStep, selectedAnimation, attacked);
                torso.render(pos, size, (int)attackCdStep, selectedAnimation, attacked);
            }
        }
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
    }
    
    @Override
    protected void moveUp() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_UP;

            this.pos.add(0, -getSpeed());
        }
    }

    @Override
    protected void moveDown() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_DOWN;

            this.pos.add(0, getSpeed());
        }
    }

    @Override
    protected void moveRight() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_RIGHT;

            this.pos.add(getSpeed(), 0);
        }
    }

    @Override
    protected void moveLeft() {
        if(!isDead()) {
            moveAnimationForward();
            selectedAnimation = ANIMATION_INDEX_LEFT;

            this.pos.add(-getSpeed(), 0);
        }
    }
    
    protected void moveAnimationForward() {
        animationCurrentStep += 1.0f * getBuffCoefficient(Buff.PercentageStat.MOVEMENT_SPEED);
        if(animationCurrentStep > 1000000.0) {
            animationCurrentStep = 0;
        }
    }

    protected float getSpeed() {
        return legs.getMovementSpeed() * getBuffCoefficient(Buff.PercentageStat.MOVEMENT_SPEED);
    }

    @Override
    public void setWaypointPath(WaypointPath waypointPath) {
        if(ai instanceof WaypointAI) {
            ((WaypointAI)ai).setWaypointPath(waypointPath);
        }
    }

    @Override
    public MoveResult[] getMoveResults() {
        int[] x = { 1, -1, 0, 0 };
        int[] y = { 0, 0, -1, 1};
        MoveType[] types = {MoveType.MOVE_RIGHT, MoveType.MOVE_LEFT, MoveType.MOVE_UP, MoveType.MOVE_DOWN};
        
        assert ((x.length == y.length) && (x.length == types.length));
        
        List<MoveResult> moveResults = new ArrayList<MoveResult>();
        
        for(int i = 0; i < x.length; i++) {
            Vector2 cpyPos = this.pos.cpy();
            cpyPos.add(getSpeed() * x[i], getSpeed() * y[i]);
            
            MoveResult temp = new MoveResult();
            temp.move = types[i];
            temp.result = cpyPos;
            moveResults.add(temp);
        }
        
        MoveResult[] out = new MoveResult[moveResults.size()];
        moveResults.toArray(out);
        return out;
    }

    @Override
    public void apply(MoveType type) {
        switch(type) {
            case MOVE_DOWN:
                tryMoveDown();
                break;
            case MOVE_LEFT:
                tryMoveLeft();
                break;
            case MOVE_RIGHT:
                tryMoveRight();
                break;
            case MOVE_UP:
                tryMoveUp();
                break;
            default:
                try {
                    throw new Exception("Missing handler for movement type " + type);
                } catch (Exception ex) {
                    Logger.getLogger(MutatedUnit.class.getName()).log(Level.SEVERE, null, ex);
                }
        }
    }

    @Override
    protected int getAttackCooldown() {
        return (int)(torso.getAttackCooldown() * getBuffCoefficient(Buff.PercentageStat.ATTACK_SPEED));
    }
    
    @Override
    public float getAttackRange() {
        return torso.getAttackRange();
    }
    
    @Override
    public float getAttackSightRange() {
        return torso.getAttackSightRange();
    }

    @Override
    protected boolean attack(AttackableEntity e) {
        if(!isDead()) {
            int[] x = { -1, 0, 1, 0 };
            int[] y = { 0, -1, 0, 1};
            Vector2 sidePos = getPos();
            sidePos.add(size.x/3 * x[selectedAnimation], size.y/3 * y[selectedAnimation]);
            
            if (torso.attack(e, this, sidePos)) {
                
                Vector2 dir = e.getPos();
                dir.sub(pos);
                float lenx = dir.x;
                if(lenx < 0) {
                    lenx *= -1.0f;
                }
                float leny = dir.y;
                if(leny < 0) {
                    leny *= -1.0f;
                }
                if(lenx > leny) {
                    if(dir.x < 0) {
                        selectedAnimation = ANIMATION_INDEX_LEFT;
                    } else {
                        selectedAnimation = ANIMATION_INDEX_RIGHT;
                    }
                } else {
                    if(dir.y < 0) {
                        selectedAnimation = ANIMATION_INDEX_UP;
                    } else {
                        selectedAnimation = ANIMATION_INDEX_DOWN;
                    }
                }
                
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    @Override
    public void getHit(int damage, AttackableEntity from) {
        if(!isDead()) {
            this.health -= damage;
            
            if(isDead()) {
                int chance = 1 + RandomGenerator.getRandom(100);
                if((float)chance / 100.0f <= BONUS_CURRENCY_DROP_CHANCE) {
                    from.getOwner().getBonusManager().addReward(this.getPos(), this.getPos().add(0.0f, size.y/2));
                }
                
                engineControls.addParticle(new BloodExplosionParticle(pos, size.x));
                
                SoundData.playRandomSound("guts_1", "guts_2");
            }
        }
    }

    @Override
    public boolean isDead() {
        if(health <= 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }
    
    @Override
    public float getDiameter() {
        return Math.min(size.x, size.y)/5.0f;
    }

    @Override
    public float getPriority() {
        return this.pos.y + size.y/3;
    }
}
