package com.pancik.libgdx.zumpatalentov.entity.unit.bodypart;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager.CostWrapper;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity;

public abstract class TorsoBodyPart implements BodyPart {
    public static enum Type {
        MELEE (new CostWrapper(10, 0)),
        ONEHAND (new CostWrapper(30, 20)),
        TWOHAND (new CostWrapper(50, 30));
        
        private ResourceManager.CostWrapper cost;

        private Type(ResourceManager.CostWrapper cost) {
            this.cost = cost;
        }

        public ResourceManager.CostWrapper getCost() {
            return cost;
        }
    }
    
    //public static Texture TORSO_MELEE_MOVE = new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_melee_move.png"));
    //public static Texture TORSO_ONEHAND_MOVE = new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_onehand_move.png"));
    //public static Texture TORSO_TWOHAND_MOVE = new Texture(Gdx.files.internal("data/drawable/unit/bodypart/body_twohand_move.png"));

    public static int TEX_SIZE_X = 256;
    public static int TEX_SIZE_Y = 256;
    
    protected Engine.EngineControls engineControls;

    public TorsoBodyPart(EngineControls engineControls) {
        this.engineControls = engineControls;
    }
    
    public abstract boolean attack(AttackableEntity defendingEntity, UnitEntity attackingEntity, Vector2 sidePos);
    public abstract int getAttackCooldown();
    public abstract float getAttackRange();
    public abstract float getAttackSightRange();
    
    public static TorsoBodyPart getTorsoBodyPart(Type type, Engine.EngineControls engineControls) {
        TorsoBodyPart out = null;
        
        switch(type) {
            case MELEE:
                out = new MeleeTorso(engineControls);
                break;
            case ONEHAND:
                out = new OneHandTorso(engineControls);
                break;
            case TWOHAND:
                out = new TwoHandTorso(engineControls);
                break;
        }
        
        return out;
    }
}
