package com.pancik.libgdx.zumpatalentov.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.math.Vector3;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.MainApplicationListener;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.ui.window.Window;
import com.pancik.libgdx.zumpatalentov.engine.ui.window.WonWindow;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonImageEnum;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ClickableButton;
import com.pancik.libgdx.zumpatalentov.ui.UIHandler;
import com.pancik.libgdx.zumpatalentov.util.InputData;
import com.pancik.libgdx.zumpatalentov.util.KeyChangedEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class SelectLevelScreen implements Screen {
    private UIHandler uiHandler;

    private static final String[] levels = {"1.txt", "2.txt", "3.txt", "4.txt", "5.txt", "6.txt", "7.txt"};
    
    private Preferences prefs = Gdx.app.getPreferences("progress");
    
    protected MainApplicationListener main;
    protected Window window = null;
    
    public SelectLevelScreen(final MainApplicationListener main) {
        this.main = main;
        SoundData.playMenuMusic();
        
        loadButtons();
    }
    
    public SelectLevelScreen(final MainApplicationListener main, boolean won) {
        this(main);
        
        boolean max = false;
        
        int progres = 0;
        if(prefs.contains("level")) {
            progres = Integer.parseInt(prefs.getString("level"));
        }
        progres += 1;
        if(progres >= levels.length) {
            progres = levels.length - 1;
            max = true;
        }
        prefs.putInteger("level", progres);
        prefs.flush();
        
        String first;
        String second;
        if(won) {
            first = "Somebody has won!";
            if(max) {
                second = "You have completed the game.";
            } else {
                second = "New level has been unlocked.";
            }
        } else {
            first = "You lost.";
            second = "Try harder next time.";
        }
        
        loadButtons();
        
        this.window = new WonWindow(first, second);
    }
    
    protected final void loadButtons() {
        final int[] widthPos = { -300, -100, +100, +300};
        int[] heightPos = { -160, 0, 160};
        String[] level = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12"};
        
        int progres = 0;
        if(prefs.contains("level")) {
            progres = Integer.parseInt(prefs.getString("level"));
        } else {
            prefs.putInteger("level", progres);
        }
        prefs.flush();
        
        this.uiHandler = new UIHandler();
        for(int j = 0; j < heightPos.length; j++) {
            for(int i = 0; i < widthPos.length; i++) {
                if(i + (j *widthPos.length) < levels.length) {
                    final int cpyJ = j;
                    final int cpyI = i;
                    this.uiHandler.addButton(new ClickableButton(ButtonHorizontalAlignment.MIDDLE,
                            widthPos[i],
                            ButtonVerticalAlignment.MIDDLE,
                            heightPos[j],
                            level[i + (j * widthPos.length)],
                            ButtonImageEnum.WH_128_128,
                            new ClickableButton.CallbackHandler() {
                                @Override
                                public void onButtonPressed() {
                                    main.setScreen(new MainGameScreen(main, "data/levels/" + levels[cpyI + (cpyJ * widthPos.length)]));
                                }
                            }, !(i + (j * widthPos.length) <= progres)));
                }
            }
        }
    }
    
    @Override
    public void tick() {
        boolean handle = true;
        
        if(window != null) {
            if(!window.isHidden()) {
                for (MouseEvent me : InputData.mouseEvents) {
                    window.handle(me);
                }
                
                Vector3 unprojectedPos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0.0f);
                main.getUICamera().unproject(unprojectedPos);
                window.tick(unprojectedPos.x, unprojectedPos.y);
                
                handle = false;
            }
        }
        
        if(handle) {
            for (MouseEvent me : InputData.mouseEvents) {
                uiHandler.handle(me);
                
                int x = RenderUtils.calculateX(ButtonHorizontalAlignment.RIGHT, -2, 32);
                int y = RenderUtils.calculateY(ButtonVerticalAlignment.TOP, 2, 32);
                if(me.mouseDown() && me.getUiUnprojectedX() > x && me.getUiUnprojectedX() < x + 32
                   && me.getGameUnprojectedY() > y && me.getUiUnprojectedY() < y + 32) {
                    if(!SoundData.SOUND_IS_ON) {
                        SoundData.SOUND_IS_ON = true;
                        SoundData.playMenuMusic();
                    } else {
                        SoundData.SOUND_IS_ON = false;
                        SoundData.stopMenuMusic();
                    }
                }
            }
        }
        
        for(KeyChangedEvent kce : InputData.keyChangedEvents) {
            if(kce.getKeyPressed() == Input.Keys.ESCAPE && kce.isDown()) {
                Gdx.app.exit();
            }
        }
    }

    @Override
    public void render() {
    }

    @Override
    public void renderUI() {
        DrawableData.renderMenuBackground();
     
        int x = RenderUtils.calculateX(ButtonHorizontalAlignment.RIGHT, -2, 32);
        int y = RenderUtils.calculateY(ButtonVerticalAlignment.TOP, 2, 32);
        if(!SoundData.SOUND_IS_ON) {
            RenderUtils.blit(DrawableData.spriteBatch,
                    DrawableData.textures.get("sound_off"),
                    new RenderUtils.ScreenInfo(x, y, 32, 32),
                    new RenderUtils.ScreenInfo(0, 0, 64, 64));
        } else {
            RenderUtils.blit(DrawableData.spriteBatch,
                    DrawableData.textures.get("sound_on"),
                    new RenderUtils.ScreenInfo(x, y, 32, 32),
                    new RenderUtils.ScreenInfo(0, 0, 64, 64));
        }
        
        uiHandler.render();

        if(window != null) {
            window.renderUI();
        }
    }

    @Override
    public void handleBackButtonPressed() {
    }

    @Override
    public void handleMenuButtonPressed() {
    }
}
