package com.pancik.libgdx.zumpatalentov.ui;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class ClickableButton {
    public static interface CallbackHandler {
        public void onButtonPressed();
    }
    
    
    protected ButtonHorizontalAlignment horizontal;
    protected int deltax;
    protected ButtonVerticalAlignment vertical;
    protected int deltay;
    protected String text;
    protected ButtonImageEnum imageType;
    protected CallbackHandler callback;
    protected boolean disabled;
    
    protected ButtonModifier modifier;

    protected boolean mouseClicked = false;
    protected Vector2 mousePos = new Vector2();

    public ClickableButton(ButtonHorizontalAlignment horizontal,
            int deltax,
            ButtonVerticalAlignment vertical,
            int deltay,
            String text,
            ButtonImageEnum imageType,
            CallbackHandler callback, boolean disabled) {
        this.horizontal = horizontal;
        this.deltax = deltax;
        this.vertical = vertical;
        this.deltay = deltay;
        this.text = text;
        this.imageType = imageType;
        this.callback = callback;
        this.modifier = ButtonModifier.NONE;
        this.disabled = disabled;
    }

    public ClickableButton(ButtonHorizontalAlignment horizontal,
            int deltax,
            ButtonVerticalAlignment vertical,
            int deltay,
            String text,
            ButtonImageEnum imageType,
            CallbackHandler callback,
            ButtonModifier modifier) {
        this(horizontal, deltax, vertical, deltay, text, imageType, callback, false);
        this.modifier = modifier;
    }

    protected int calculateX() {
        return RenderUtils.calculateX(horizontal, deltax, imageType.getDensitySizex());
    }

    protected int calculateY() {
        return RenderUtils.calculateY(vertical, deltay, imageType.getDensitySizey());
    }

    public boolean checkIfClicked(float x, float y) {
        if (calculateX() < x && calculateX() + imageType.getDensitySizex() > x && calculateY() < y && calculateY() + imageType.getDensitySizey() > y) {
            return true;
        } else {
            return false;
        }
    }

    public void click() {
        callback.onButtonPressed();
    }

    public void handle(MouseEvent me) {
        if(!disabled) {
            if(me.mouseDown()) {
                mouseClicked = true;
                mousePos.x = me.getUiUnprojectedX();
                mousePos.y = me.getUiUnprojectedY();
            } else if(me.mouseDragged()) {
                mousePos.x = me.getUiUnprojectedX();
                mousePos.y = me.getUiUnprojectedY();
            } else if(me.mouseUp()) {
                if(mouseClicked) {
                    mouseClicked = false;
                    if(checkIfClicked(mousePos.x, mousePos.y)) {
                        this.click();
                    }
                }
            }
        }
    }

    protected void renderClicked() {
        RenderUtils.blit(DrawableData.spriteBatch,
                             DrawableData.textures.get(imageType.getClickedTextName()),
                             new RenderUtils.ScreenInfo(calculateX(), calculateY(), imageType.getDensitySizex(), imageType.getDensitySizey()),
                             new RenderUtils.ScreenInfo(imageType.getClickedTextBegx(), imageType.getClickedTextBegy(), imageType.getClickedTextEndx(), imageType.getClickedTextEndy()));
    }
    
    public void render() {
        int textDeltay = 0;

        if((!checkIfClicked(mousePos.x, mousePos.y) || !mouseClicked) && !disabled) {
            RenderUtils.blit(DrawableData.spriteBatch,
                             DrawableData.textures.get(imageType.getTextName()),
                             new RenderUtils.ScreenInfo(calculateX(), calculateY(), imageType.getDensitySizex(), imageType.getDensitySizey()),
                             new RenderUtils.ScreenInfo(imageType.getTextBegx(), imageType.getTextBegy(), imageType.getTextEndx(), imageType.getTextEndy()));
        } else {
            renderClicked();
            textDeltay = 1;
        }

        RenderUtils.blitText(DrawableData.spriteBatch,
                             this.text,
                             calculateX() + ((imageType.getDensitySizex()) / 2),
                             calculateY() + ((imageType.getDensitySizey()) / 2) + textDeltay,
                             RenderUtils.TextAlligment.MIDDLE,
                             RenderUtils.TextAlligment.MIDDLE);

        switch(modifier) {
            case NONE:
                break;

            case DONE:
                //DrawableData.whiteFont.setScale(0.65f);
                //DrawableData.blackFont.setScale(0.65f);

                RenderUtils.blitText(DrawableData.spriteBatch,
                             "Done",
                             calculateX() + ((imageType.getDensitySizex()) / 2),
                             calculateY() + (imageType.getDensitySizey()/2) + (imageType.getDensitySizey()/4) + 1 + textDeltay,
                             Color.GREEN,
                             RenderUtils.TextAlligment.MIDDLE,
                             RenderUtils.TextAlligment.MIDDLE);

                DrawableData.setFontScale(1.0f);
                DrawableData.setFontScale(1.0f);
                break;

            case COMING_SOON:
                String modifierText = "Coming soon";

                DrawableData.setFontScale(0.65f);
                DrawableData.setFontScale(0.65f);

                BitmapFont.TextBounds tb = DrawableData.whiteFont.getBounds(modifierText);
                float modifierDeltaX = tb.width / 2;
                float modifierDeltaY = tb.height / 2;

                DrawableData.whiteFont.draw(DrawableData.spriteBatch,
                        modifierText,
                        calculateX() + ((imageType.getDensitySizex()) / 2) + 2 - modifierDeltaX,
                        calculateY() + (imageType.getDensitySizey()/2) + (imageType.getDensitySizey()/4) + 2 - modifierDeltaY + textDeltay);

                DrawableData.blackFont.setColor(Color.RED);
                DrawableData.blackFont.draw(DrawableData.spriteBatch,
                        modifierText,
                        calculateX() + ((imageType.getDensitySizex()) / 2) - modifierDeltaX,
                        calculateY() + (imageType.getDensitySizey()/2) + (imageType.getDensitySizey()/4) + 2 - modifierDeltaY + textDeltay);

                DrawableData.blackFont.setColor(Color.BLACK);

                DrawableData.setFontScale(1.0f);
                DrawableData.setFontScale(1.0f);

                break;
            case EMPTY:
                modifierText = "Empty";

                DrawableData.setFontScale(0.75f);
                DrawableData.setFontScale(0.75f);

                tb = DrawableData.whiteFont.getBounds(modifierText);
                modifierDeltaX = tb.width / 2;
                modifierDeltaY = tb.height / 2;

                DrawableData.whiteFont.draw(DrawableData.spriteBatch,
                        modifierText,
                        calculateX() + ((imageType.getDensitySizex()) / 2) + 2 - modifierDeltaX,
                        calculateY() + (imageType.getDensitySizey()/2) + (imageType.getDensitySizey()/4) + 2 - modifierDeltaY + textDeltay);

                DrawableData.blackFont.setColor(Color.RED);
                DrawableData.blackFont.draw(DrawableData.spriteBatch,
                        modifierText,
                        calculateX() + ((imageType.getDensitySizex()) / 2) - modifierDeltaX,
                        calculateY() + (imageType.getDensitySizey()/2) + (imageType.getDensitySizey()/4) + 2 - modifierDeltaY + textDeltay);

                DrawableData.blackFont.setColor(Color.BLACK);

                DrawableData.setFontScale(1.0f);
                DrawableData.setFontScale(1.0f);

                break;
            case RECOMMENDED:
                modifierText = "Recommended";

                DrawableData.setFontScale(0.80f);
                DrawableData.setFontScale(0.80f);

                tb = DrawableData.whiteFont.getBounds(modifierText);
                modifierDeltaX = tb.width / 2;
                modifierDeltaY = tb.height / 2;

                DrawableData.whiteFont.draw(DrawableData.spriteBatch,
                        modifierText,
                        calculateX() + ((imageType.getDensitySizex()) / 2) + 2 - modifierDeltaX,
                        calculateY() + (imageType.getDensitySizey()/2) + (imageType.getDensitySizey()/4) + 2 - modifierDeltaY + textDeltay);

                DrawableData.blackFont.setColor(new Color(0.0f, 0.5f, 0.0f, 1.0f));
                DrawableData.blackFont.draw(DrawableData.spriteBatch,
                        modifierText,
                        calculateX() + ((imageType.getDensitySizex()) / 2) - modifierDeltaX,
                        calculateY() + (imageType.getDensitySizey()/2) + (imageType.getDensitySizey()/4) + 2 - modifierDeltaY + textDeltay);

                DrawableData.blackFont.setColor(Color.BLACK);

                DrawableData.setFontScale(1.0f);
                DrawableData.setFontScale(1.0f);

                break;
        }
    }

    public void setText(String text) {
        this.text = text;
    }

    public void setModifier(ButtonModifier modifier) {
        this.modifier = modifier;
    }

    public ButtonModifier getModifier() {
        return modifier;
    }
}
