package com.pancik.libgdx.zumpatalentov.util;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.MainApplicationListener;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;

public final class RenderUtils {
    public static enum TextAlligment {
        LEFT, MIDDLE, RIGHT, TOP, BOTTOM;
    }

    public static class ScreenInfo {
        int posx, posy, sizex, sizey;

        public ScreenInfo(int posx, int posy, int sizex, int sizey) {
            this.posx = posx;
            this.posy = posy;
            this.sizex = sizex;
            this.sizey = sizey;
        }

        public ScreenInfo(float posx, float posy, float sizex, float sizey) {
            this.posx = (int)posx;
            this.posy = (int)posy;
            this.sizex = (int)sizex;
            this.sizey = (int)sizey;
        }
    }

    public static int calculateX(ButtonHorizontalAlignment horizontal, int deltax, int sizex) {
        int width = Gdx.graphics.getWidth();
        if(MainApplicationListener.smallScreen) {
            width = 800;
        }

        switch(horizontal) {
            case LEFT:
                return (int)(deltax * DrawableData.getDensity());
            case MIDDLE:
                return (width/2) + (int)(deltax * DrawableData.getDensity()) - sizex/2;
            case RIGHT:
                return width + (int)(deltax * DrawableData.getDensity()) - sizex;
        }
        return -1;
    }

    public static int calculateY(ButtonVerticalAlignment vertical, int deltay, int sizey) {
        int height = Gdx.graphics.getHeight();
        if(MainApplicationListener.smallScreen) {
            height = 480;
        }

        switch(vertical) {
            case TOP:
                return (int)(deltay * DrawableData.getDensity());
            case MIDDLE:
                return (height/2) + (int)(deltay * DrawableData.getDensity()) - sizey/2;
            case BOTTOM:
                return height + (int)(deltay * DrawableData.getDensity())  - sizey;
        }
        return -1;
    }
    
    public static void blit(SpriteBatch sb, Texture what, ScreenInfo screenPos, ScreenInfo textureDesc) {
        sb.draw(what, screenPos.posx, screenPos.posy, screenPos.sizex, screenPos.sizey,
                textureDesc.posx, textureDesc.posy, textureDesc.sizex, textureDesc.sizey, false, true);
    }
    
    public static void blit(SpriteBatch sb, Texture what, ScreenInfo screenPos, ScreenInfo textureDesc, float rotation) {
        sb.draw(what, screenPos.posx, screenPos.posy, screenPos.sizex/2, screenPos.sizey/2, screenPos.sizex, screenPos.sizey, 1, 1, rotation,
                textureDesc.posx, textureDesc.posy, textureDesc.sizex, textureDesc.sizey, false, true);
    }

    public static void blitText(SpriteBatch sb, CharSequence str, int x, int y, TextAlligment horizontal, TextAlligment vertical) {
        BitmapFont.TextBounds tb = DrawableData.whiteFont.getBounds(str);
        float deltax = 0;
        float deltay = 0;

        switch(horizontal) {
            case LEFT:
                deltax = 0;
                break;
            case MIDDLE:
                deltax = tb.width/2;
                break;
            case RIGHT:
                deltax = tb.width;
                break;
        }

        switch(vertical) {
            case TOP:
                deltay = tb.height;
                break;
            case MIDDLE:
                deltay = tb.height/2;
                break;
            case BOTTOM:
                deltay = 0;
                break;
        }

        DrawableData.whiteFont.draw(DrawableData.spriteBatch, str, x + 2 - deltax, y + 2 - deltay);
        DrawableData.blackFont.draw(DrawableData.spriteBatch, str, x - deltax, y - deltay);
    }

    public static void blitText(SpriteBatch sb, CharSequence str, int x, int y, Color color,TextAlligment horizontal, TextAlligment vertical) {
        BitmapFont.TextBounds tb = DrawableData.whiteFont.getBounds(str);
        float deltax = 0;
        float deltay = 0;

        switch(horizontal) {
            case LEFT:
                deltax = 0;
                break;
            case MIDDLE:
                deltax = tb.width/2;
                break;
            case RIGHT:
                deltax = tb.width;
                break;
        }

        switch(vertical) {
            case TOP:
                deltay = tb.height;
                break;
            case MIDDLE:
                deltay = tb.height/2;
                break;
            case BOTTOM:
                deltay = 0;
                break;
        }

        if(color != Color.WHITE) {
            DrawableData.whiteFont.draw(DrawableData.spriteBatch, str, x + 2 - deltax, y - deltay);
        }
        DrawableData.blackFont.setColor(color);
        DrawableData.blackFont.draw(DrawableData.spriteBatch, str, x - deltax, y - deltay);
        DrawableData.blackFont.setColor(Color.BLACK);
    }

    public static void blitText(SpriteBatch sb,
            CharSequence str,
            ButtonHorizontalAlignment horizontal,
            int deltax,
            ButtonVerticalAlignment vertical,
            int deltay,
            TextAlligment textHorizontal,
            TextAlligment textVertical) {

        BitmapFont.TextBounds tb = DrawableData.whiteFont.getBounds(str);
        float textDeltax = 0;
        float textDeltay = 0;

        switch(textHorizontal) {
            case LEFT:
                textDeltax = 0;
                break;
            case MIDDLE:
                textDeltax = tb.width/2;
                break;
            case RIGHT:
                textDeltax = tb.width;
                break;
        }

        switch(textVertical) {
            case TOP:
                textDeltay = tb.height;
                break;
            case MIDDLE:
                textDeltay = tb.height/2;
                break;
            case BOTTOM:
                textDeltay = 0;
                break;
        }

        int calculatedX = calculateX(horizontal, deltax, 0);
        int calculatedY = calculateY(vertical, deltay, 0);

        DrawableData.whiteFont.draw(DrawableData.spriteBatch, str, calculatedX + 2 - textDeltax, calculatedY + 2 - textDeltay);
        DrawableData.blackFont.draw(DrawableData.spriteBatch, str, calculatedX - textDeltax, calculatedY - textDeltay);
    }

    public static void blitText(SpriteBatch sb,
            CharSequence str,
            ButtonHorizontalAlignment horizontal,
            int deltax,
            ButtonVerticalAlignment vertical,
            int deltay,
            TextAlligment textHorizontal,
            TextAlligment textVertical,
            Color color, float colorAlpha) {

        BitmapFont.TextBounds tb = DrawableData.whiteFont.getBounds(str);
        float textDeltax = 0;
        float textDeltay = 0;

        switch(textHorizontal) {
            case LEFT:
                textDeltax = 0;
                break;
            case MIDDLE:
                textDeltax = tb.width/2;
                break;
            case RIGHT:
                textDeltax = tb.width;
                break;
        }

        switch(textVertical) {
            case TOP:
                textDeltay = tb.height;
                break;
            case MIDDLE:
                textDeltay = tb.height/2;
                break;
            case BOTTOM:
                textDeltay = 0;
                break;
        }

        int calculatedX = calculateX(horizontal, deltax, 0);
        int calculatedY = calculateY(vertical, deltay, 0);
        
        if(color != Color.WHITE) {
            DrawableData.whiteFont.setColor(1.0f, 1.0f, 1.0f, colorAlpha);
            DrawableData.whiteFont.draw(DrawableData.spriteBatch, str, calculatedX + 2 - textDeltax, calculatedY + 2 - textDeltay);
        }
        DrawableData.blackFont.setColor(color.r, color.g, color.b, colorAlpha);
        DrawableData.blackFont.draw(DrawableData.spriteBatch, str, calculatedX - textDeltax, calculatedY - textDeltay);
        
        DrawableData.blackFont.setColor(Color.BLACK);
        DrawableData.whiteFont.setColor(Color.WHITE);
    }
    
    public static void blitText(SpriteBatch sb,
            CharSequence str,
            ButtonHorizontalAlignment horizontal,
            int deltax,
            ButtonVerticalAlignment vertical,
            int deltay,
            TextAlligment textHorizontal,
            TextAlligment textVertical,
            Color color) {
        blitText(sb, str, horizontal, deltax, vertical, deltay, textHorizontal, textVertical, color, color.a);
    }
}